/******************************************************************************
File:    C_8530.H
Project: Serial controller developers library
Purpose: Serial channel hardware description (SCC 8530 USART).
Company: Comtrol Corporation
Author:  Craig Harrison
*******************************************************************************
Copyright 1993 Comtrol Corporation. All rights reserved.
Subject to developers license agreement.
******************************************************************************/

#ifndef C_8530
#define C_8530 1

/******************************************************************************
                        SCC Register Index Defines
******************************************************************************/
#define WR0      0                     /* write registers */
#define WR1      1
#define WR2      2
#define WR3      3
#define WR4      4
#define WR5      5
#define WR6      6
#define WR7      7
#define WR8      8
#define WR9      9
#define WR10     10
#define WR11     11
#define WR12     12
#define WR13     13
#define WR14     14
#define WR15     15

#define RR0      0                     /* read registers */
#define RR1      1
#define RR2      2
#define RR3      3
#define RR8      8
#define RR10     10
#define RR12     12
#define RR13     13
#define RR15     15

/******************************************************************************
                     Baud Rate Generator Time Constants
Uses x16 Baud Rate Factor, based on a 4.9152 MHz clock.  Other factors or
clocks are possible.
*******************************************************************************
*/
#define BPS50           3070
#define BPS75           2046
#define BPS110          1394           /* 0.026 percent error */
#define BPS134          1140           /* 0.001 percent error */
#define BPS150          1022
#define BPS300          510
#define BPS600          254
#define BPS1200         126
#define BPS1800         83             /* 0.401 percent error */
#define BPS2000         75             /* 1.06 percent error */
#define BPS2400         62
#define BPS3600         41             /* 1.62 percent error */
#define BPS4800         30
#define BPS7200         19             /* 1.75 percent error */
#define BPS9600         14
#define BPS19200        6
#define BPS38400        2
#define BPS56000        1              /* 74.3 percent error */
#define BPS76800        0

/******************************************************************************
                     SCC Write Register Contents Defines
******************************************************************************/
/* Write Register 0 - command register */
#define RESET_EXT        0x10
#define RESET_TX_INT     0x28
#define RESET_ERROR      0x30
#define RESET_IUS        0x38

/* Write Register 1 - Tx/Rx interrupt and data transfer mode definition */
#define EXT_INT_ENABLE   0x01
#define TX_INT_ENABLE    0x02
#define PARITY_SPECIAL   0x04
#define RX_INT_ENABLE    0x10

/* Write Register 2 - interrupt vector */

/* Write Register 3 - Rx parameters and controls */
#define RX_ENABLE        0x01
#define RX5_BIT_CHAR     0x00
#define RX7_BIT_CHAR     0x40
#define RX6_BIT_CHAR     0x80
#define RX8_BIT_CHAR     0x0c0


/* Write Register 4 - Tx/Rx miscellaneous parameters and modes */
#define PARITY_ENABLE    0x01
#if QNX
#define cPARITY_EVEN     0x02
#define cPARITY_ODD      0x00
#else
#define PARITY_EVEN      0x02
#define PARITY_ODD       0x00
#endif
#define ONE_STOP_BIT     0x04
#define ONEH_STOP_BITS   0x08
#define TWO_STOP_BITS    0x0c
#define X16_CLOCK        0x40

/* Write Register 5 - Tx paramerameters and controls */
#define RTS              0x02
#define TX_ENABLE        0x08
#define BREAK            0x10
#define TX5_BIT_CHAR     0x00
#define TX7_BIT_CHAR     0x20
#define TX6_BIT_CHAR     0x40
#define TX8_BIT_CHAR     0x60
#define DTR              0x80

/* Write Register 6 - sync character or SDLC address field */

/* Write Register 7 - sync character or SDLC flag */

/* Write Register 8 - transmit buffer */

/* Write Register 9 - master interrupt control */
#define VIS              0x01
#define NV               0x02
#define DLC              0x04
#define MIE              0x08
#define STATUS_LO        0x00
#define STATUS_HI        0x10
#define RESET_CH_B       0x40
#define RESET_CH_A       0x80
#define HARDWARE_RESET   0xc0

/* Write Register 10 - miscellaneous Tx/Rx control bits */

/* Write Register 11 - clock mode control */
#define TX_CLK_EQ_BRG    0x10
#define RX_CLK_EQ_BRG    0x40

/* Write Register 12 - lower byte of baud generator time constant */

/* Write Register 13 - upper byte of baud generator time constant */

/* Write Register 14 - miscellaneous control bits */
#define BRG_ENABLE       0x01
#define BRG_EQ_SYS_CLK   0x02

/* Write Register 15 - external/STATUS interrupt control */
#define CD_IE            0x08
#define SYNCHUNT_IE      0x10
#define CTS_IE           0x20
#define TXUNDRUN_IE      0x40
#define BKABORT_IE       0x80

/******************************************************************************
                     SCC Read Register Contents Defines
******************************************************************************/
/* Read Register 0 - Tx/Rx BUFFER STATUS and external STATUS */
#define RX_BUFFER       0x01
#define TX_BUFFER       0x04
#define CD              0x08
#define SYNCHUNT        0x10
#define CTS             0x20
#define TXUNDRUN        0x40
#define BKABORT         0x80

/* Read Register 1 - special receive condition STATUS */
#define ALLSENT         0x01
#define PARITY_ERR      0x10
#define OVRRUN_ERR      0x20
#define FRAME_ERR       0x40
#define EOFRM           0x80

/* Read Register 2 - interrupt vector */

/* Read Register 3 - interrupt pending STATUS */

/* Read Register 8 - receive data register */

/* Read Register 10 - miscellaneous STATUS bits */

/* Read Register 12 - value stored in WR12 */

/* Read Register 13 - value stored in WR13 */

/* Read Register 15 - value stored in WR15 */

#endif /* #ifndef C_8530 */
  
