/*
*****************************************************************************
File:    S_SSCI.H
Project: API/SSCI Libraries
Purpose: Library header file.
Company: Comtrol Corporation
Author:  Jerry Medin
*****************************************************************************
Copyright 1993 Comtrol Corporation. All rights reserved.
Subject to developers license agreement.
*****************************************************************************
*/

#if !QNX
#include <dos.h>
#include <memory.h>
#endif
#include <conio.h>
#include <string.h>

#if QNX
#include <i86.h>
#include <sys/sched.h>
#endif

#ifndef S_API
#define S_API 1

/*
*****************************************************************************
  Error numbers 
*****************************************************************************
*/
#define NUM_ERRMSG 2          /* number of error messages */
#define ERR_NULL 0
#define ERR_BADWINSIZE 1
#define ERRMSGSIZE 80         /* maximum length of error messages */


/*
*****************************************************************************
  Controller IDs 
*****************************************************************************
*/
#define CTLID_H186     0x00     /* Hostess 186 standard 4/8 port */
#define CTLID_HSTI     0x01     /* Hostess i */
#define CTLID_HISPDS   0x02     /* Hi Speed Sync */
#define CTLID_DUALS    0x03     /* Intelligent Dual Sync */
#define CTLID_H186RJ45 0x04     /* Hostess 186 4 port RJ 45 */ 
#define CTLID_H186RJ11 0x05     /* Hostess 186 8 port RJ 11 */
#define CTLID_H18616   0x06     /* Hostess 186 16 port */
#define CTLID_HSTIMC   0x6d     /* Hostess i Micro Channel */
#define CTLID_NONE     0xff     /* ID not supported by this controller */



/*
*****************************************************************************
  Descriptors and defines for a controller, DPM windows, and I/O registers 
*****************************************************************************
*/

#ifdef SMARTH
#define IOINOUT               /* I/O address offset to In/Out registers */
#define IOSECURITY 1          /* I/O address offset to security PAL */
#define IOINT 2               /* I/O address offset to interrupt CTLP */
#define IORESET 3             /* I/O address offset to reset CTLP */      
#define CTLRESET_ASSERT 0     /* I/O value to assert CTLP reset line */
#define CTLRESET_NEGATE 0xff  /* I/O value to negate CTLP reset line */
#define CTLINT 0              /* I/O value to interrupt CTLP */
#endif


#ifdef HOSTESSi
#define IOCTLIDLO 2           /* I/O address offset to ctrl'r ID register LSB */
#define IOCTLIDHI 3           /* I/O address offset to ctrl'r ID register MSB */
#define IOCR 0                /* I/O address offset to control registers */
#define IOX 1                 /* I/O address offset to ctl reg index */
#define IOINT 2               /* I/O address offset to interrupt CTLP */
#define IORESET 3             /* I/O address offset to reset CTLP */      
#define CR1X 0x01             /* control register 1 index */
#define CR2X 0x02             /* control register 2 index */
#define CR3X 0x08             /* control register 3 index */
#define CR4X 0x10             /* control register 4 index */
#define CTLRESET_ASSERT 0     /* I/O value to assert CTLP reset line */
#define CTLRESET_NEGATE 0xff  /* I/O value to negate CTLP reset line */
#define CTLINT 0              /* I/O value to interrupt CTLP */
#define EN_DPM   0            /* I/O value to enable DPM */  
#define DIS_DPM 0x04          /* I/O value to disable DPM */
#define BUSSIZE16 0x80        /* I/O value for 8 bit data bus */
#define DIS_SLIDE 0x00        /* Can't disable sliding window on           */
                              /* Hostess/i, but this avoids a compiler     */
                              /* error when compiling for different boards */
#endif


#ifdef HOSTESS186
#define IOCTLIDLO 2           /* I/O address offset to ctrl'r ID register LSB */
#define IOCTLIDHI 3           /* I/O address offset to ctrl'r ID register MSB */
#define IOCR 0                /* I/O address offset to control registers */
#define IOX 1                 /* I/O address offset to ctl reg index */
#define IOINT 2               /* I/O address offset to interrupt CTLP */
#define IORESET 3             /* I/O address offset to reset CTLP */      
#define CR1X 0x01             /* control register 1 index */
#define CR2X 0x02             /* control register 2 index */
#define CR3X 0x04             /* control register 3 index */
#define CR4X 0x08             /* control register 4 index */
#define CTLRESET_ASSERT 0     /* I/O value to assert CTLP reset line */
#define CTLRESET_NEGATE 0xff  /* I/O value to negate CTLP reset line */
#define CTLINT 0              /* I/O value to interrupt CTLP */
#define EN_DPM   0x10         /* I/O value to enable DPM */  
#define DIS_DPM 0             /* I/O value to disable DPM */
#define BUSSIZE16 0x20        /* I/O value for 8 bit data bus */
#define DIS_SLIDE 0x10        /* I/O value to disable sliding window */
#endif

/*
*****************************************************************************
  SSCI Defines   
*****************************************************************************
*/

#define NUM_CTLRS   8  /* Number of controllers allowed in a system */
#define LNTBL_SIZE  16 /* Number of lines per controller (usually 8 or 16) */
#define SMTBL_SIZE  25 /* Number of event functions in the System Message Table */

/*
*****************************************************************************
  MS-Windows  Defines
*****************************************************************************
*/

#define TIMER_SEC 1
#define WM_AA_RCV_DATA WM_USER+100    /* Receive data waiting for a line */
#define WM_AA_BREAK    WM_USER+101    /* A Break character was received  */
#define WM_AA_MDM_CHG  WM_USER+102    /* A Modem change occurred on a line */
#define WM_AA_RCV_ERR  WM_USER+103    /* A Receive error occurred on a line */


/*
*****************************************************************************
  Firmware data area that exists on the controller
*****************************************************************************
*/
#define FIRMWARE_AREA  0x00000b80



typedef struct                /* descriptor for one SYSP window into DPM */
{
   UBYTE_T CtlReg;              /* window control register value */
   ULONG_T Offset;            /* offset of this window from start of DPM */
} DPWINDOW_T;

typedef struct                /* descriptor for all SYSP windows into DPM */
{

   WORD_T CurWin;               /* current window number */
   WORD_T DefWin;               /* default window number, always return here after
                                 moving the window */
   WORD_T NumWin;               /* number of windows available */
   WORD_T WinSizeKB;            /* window size in K bytes */
   WORD_T DPMDiv;               /* divisor of DPM offset to convert to window num */
#if BORLANDC && WINDAPI
   DPWINDOW_T far *WinArray;      /* array of descriptors for each window */
#else
   DPWINDOW_T *WinArray;      /* array of descriptors for each window */
#endif
} DPWINDOWS_T;

typedef struct                /* control register images */
{
   UBYTE_T CR1;
   UBYTE_T CR2;
   UBYTE_T CR3;
   UBYTE_T CR4;
} CTLREGS_T;

typedef struct                /* Descriptor for an instance of a controller */
{
   int NumLines;              /* number of lines on this controller */
   int Io;                    /* base I/O address */
   long DPMAbsolute;          /* DPM base address (as absolute address) */
   char far *DPM;             /* DPM base address (as pointer) */
   int DPMSizeKB;             /* DPM size in K bytes */
   int IRQNum;                /* interrupt request number */
   int BusSize;               /* data bus size (8 or 16 bit) */
   int WinEnable;             /* TRUE if window sliding enabled, else FALSE */
   CTLREGS_T CReg;            /* control registers */
   DPWINDOWS_T DPWin;         /* SYSP windows into DPM */
   int FirstLine;             /* first line number on controller */
   FIXDPM_T far *FixDPMP;		 /* ptr to fixed DPM area */
   FIRMUSER_T far *fup;       /* pointer to firmware data area */

   IPINT_T far *CtlIntP;      /* pointer to CTLP interrupt desc */  
   IPINT_T far *SysIntP;      /* pointer to SYSP interrupt desc */

   MSGQ_T far *CtlQP;			 /* ptr to controller queue */
   MSGQ_T far *SysQP;			 /* ptr to system queue */

   IFG_T far *CtlIFGP;        /* ptr to CTLP int flag group desc */
   IFG_T far *SysIFGP;        /* ptr to SYSP int flag group desc */
   UWORD_T far *StatusFlagGrp;/* status flag group array */
   BYTE_T far *CtlDPAccP;     /* ptr to CTLP dual port access request */
   BYTE_T far *SysDPAccP;     /* ptr to SYSP dual port access request */
   UBYTE_T far *CtlReplyStat; /* CTLP reply status array */
   REPLYBUF_T far *CtlReply;  /* CTLP reply buffer array */
   UBYTE_T far *SysReplyStat; /* SYSP reply status array */
   REPLYBUF_T far *SysReply;  /* SYSP reply buffer array */
   DBCB_T far *SDBTbl;        /* SDB table array */
   WORD_T far *SDBFreeP;      /* ptr to index of 1st shared DBCB in free pool */
   WORD_T far *SDBFreeCntP;   /* number SDBs on free pool */

   LINE_T far * far *Line;    /* ptrs to line tables */
   WORD_T Selector;           /* Selector for Microsoft Windows */
#if WINDAPI
   LPSTR  WinSelPtr;          /* Selector Ptr for Microsoft Windows */
#endif
#if QNX
   char far *WinSelPtr;       /* Selector Ptr for Microsoft Windows */
#endif
} CTL_T;


/*
 *****************************************************************************
  Define a type to hold pointers to data in the SSCI data space.
 *****************************************************************************
*/
typedef struct
{
  int far *sNumCtlP;      
  CTL_T far *sCtlArrP;
} CRITPTR_T;

/*
 *****************************************************************************
  Define a type to hold windows messages that PostMessage can't take.
 *****************************************************************************
*/
#define POSTQ_SIZE 128
typedef struct
{
  int Dev;     
  unsigned int Type;
  long lParam; 
} POSTMSG_T;

typedef struct
{
  int In;      
  int Out;     
  POSTMSG_T PostMsg[POSTQ_SIZE];     
} POSTQ_T;

/*
******************************************************************************
  Define a type that holds all Receive event counts for all lines
******************************************************************************
*/

typedef struct            
{
  int MdmState;
  int BreakCount;
  int OverrunCount;
  int ParityErrCount;
  int FrameErrCount;
} EV_RCV_T;

/*
 *****************************************************************************
  Define a type to hold PIC values for each IRQ.
 *****************************************************************************
*/

typedef struct                /* IRQ values form interrupt service routines */
{
   int Mask;
   int Vector;
   int EOIPort;
} PIC_T;


/*
 *****************************************************************************
  Define struture to hold configuration file parameters.
 *****************************************************************************
*/

typedef struct     
{
  unsigned int IoAddr;     /* port I/O address */
  unsigned long BaseAddr;  /* DPM address */
  unsigned int Irq;        /* IRQ number */
  unsigned int BusSize;    /* bus size 8 or 16 bits */
  unsigned long CPStartAddr;/* control program starting address */
  char TdFlag;             /* Turbo Debug flag */
  char CtlPgmName[128];    /* name of control program to download */
  unsigned int NumLines;   /* number of lines on this controller */
} CFCTL_T;


/*
 *****************************************************************************
  Define struture to hold configuration for all controllers.
 *****************************************************************************
*/
typedef struct     
{
  int NumCtlrs;              /* Number of controllers defined in file */
  CFCTL_T CfCtl[8];
} CF_T;



#define HiCR0       0x01                    /* Select Control Reg 0 */
#define HiCR1       0x02                    /* Select Control Reg 1 */
#define HiCR2       0x08                    /* Select Control Reg 2 */
#define HiCR3       0x10                    /* Select Control Reg 3 */

#ifdef HOSTESSi
#define EN_RAM    0x00                      /* Enable DPRAM */
#define DIS_RAM   0x04                      /* Disable DPRAM */
#endif

#ifdef HOSTESS186
#define EN_RAM    0x80                      /* Enable DPRAM */
#define DIS_RAM   0x00                      /* Disable DPRAM */
#endif

#define EOI  0x20
#define EXEC 0x01
#define COPY 0x02




/* Consumer processing status of an IFG. These are the status bytes returned
   by IFGStatus(). */
#define IFGSTAT_IDLE      0   /* IFG not pending or being processed */
#define IFGSTAT_PEND      1   /* an IFG is pending, flags have been set but
                                 the consumer has not yet begun processing */ 
#define IFGSTAT_PROCESS   2   /* IFG being processed by consumer */

/*
*****************************************************************************
  Controller ID values for various controllers
*****************************************************************************
*/

#define ID_HOSTESS186STD   0x00       /* Hostess 186 standard 4/8 port */
#define ID_HOSTESSiRED     0x01       /* Hostess i Redesign (Rev ___) */
#define ID_HISPEEDSYNC     0x02       /* Hi Speed Sync */
#define ID_HOSTESSi_2      0x03       /* Intelligent Dual Sync */
#define ID_HOSTESS186RJ45  0x04       /* Hostess 186 4 port RJ 45 */
#define ID_HOSTESS186RJ11  0x05       /* Hostess 186 8 port RJ 11 */
#define ID_HOSTESSi_MC     0x6d       /* Hostess i Micro Channel */
#define ID_HOSTESS186_16   0x06       /* Hostess 186 16 port */


/*
*****************************************************************************
  Function Prototypes for sl_ssci.c
*****************************************************************************
*/
#if WINDAPI
int FAR PASCAL sControllerExists(int);
void FAR PASCAL sInitCriticalPtrs(CRITPTR_T far *);
BOOL_T FAR PASCAL sSendCtlQMsg(int,void far *, void far *,int);
int FAR PASCAL sGetRxData(int,LINE_T far *, int, char far *);
int FAR PASCAL sPeekRx(LINE_T far *);
int FAR PASCAL sPutTxData(int,LINE_T far *, int, char far *);
int FAR PASCAL sPutTxBlocks(int,LINE_T far *, int, char far *, int, char far *, int, char far *);
void FAR PASCAL sCleanupRx(int,int);
void FAR PASCAL sInit(int,int);
void FAR PASCAL sEvIFGNull(int,unsigned int);
void FAR PASCAL sStartCP(int,int);
void FAR PASCAL sInitCtlDesc(int,int,long,int,int,int,int,int,int,int,int);
void FAR PASCAL sInstallEvIFG(int, WORD_T, void (FAR PASCAL *)(int,unsigned int));
void FAR PASCAL sInstallEvSM(int, WORD_T, void (FAR PASCAL *)(int, void far *));
void FAR PASCAL sEvSM00(int,void far *);
void FAR PASCAL sEvSM01(int,void far *);
void FAR PASCAL sHookIsr(int);
void FAR PASCAL sUnHookIsr(void);
void FAR PASCAL sDelay (int);
WORD_T FAR PASCAL sGetCtlQCnt(int);
WORD_T FAR PASCAL sGetSysQCnt(int);
void FAR PASCAL sSetNumLines(int,int);
unsigned long FAR PASCAL sGetCtlTime(int);
UBYTE_T FAR PASCAL sGetIFGStat(int, BYTE_T,UWORD_T far *);
unsigned int FAR PASCAL sGetCtlID(int);
int FAR PASCAL sGetNumLines(int);
UWORD_T FAR PASCAL sGetSFG(int,BYTE_T,UWORD_T);
void FAR PASCAL sSetIFG(int,BYTE_T,UWORD_T);
UWORD_T FAR PASCAL sDecrementSFG(int,BYTE_T);
UWORD_T FAR PASCAL sIncrementSFG(int,BYTE_T);
UWORD_T FAR PASCAL sTestSetSFG(int,BYTE_T,UWORD_T);
void FAR PASCAL sClrSFG(int,BYTE_T,UWORD_T);
void FAR PASCAL sInitSFG(int,BYTE_T,UWORD_T);
void FAR PASCAL sSetSFG(int,BYTE_T,UWORD_T);

LINE_T far *sLnNumToLnPtr(int,int);


void sFreeGPSDB(int,int);
void sFreeTxSDB(int,LINE_T *,WORD_T);
void sFreeRxSDB(int,LINE_T *,WORD_T);
WORD_T sSDBAddrToWindowNumber(int,WORD_T);
#if BORLANDC && WINDAPI
void sInitWinDesc(DPWINDOWS_T far *,WORD_T,WORD_T,WORD_T);
#else
void sInitWinDesc(DPWINDOWS_T *,WORD_T,WORD_T,WORD_T);
#endif
char far *sMemAbs2Ptr(unsigned long);
void sInitCtlRegHSTi(int);
void sInitCtlRegH186(int);
void sCtlReset(int);
void sCtlInterrupt(int);
void sSlideWindow(int,int);
void __interrupt __far IntHandler();
ULONG_T sFarPtrToAbs(void far *);
void far *sAbsToFarPtr(ULONG_T);
void sGetSysQMsg(int);
void sGetDPAccess(int);
void sIFGSet(int,BYTE_T,UWORD_T);
void sDispatchIFG(int,int);
WORD_T sAllocGPSDB(int);
void sEnqSDB(int,WORD_T far *,WORD_T);
WORD_T sDeqSDB(int,WORD_T far *);
WORD_T sAllocTxSDB(int,LINE_T far *);
WORD_T sAllocRxSDB(int,LINE_T far *);
int sGetRxSDBIndex(int,int);
int sGetReplyID(int);
int sWaitForReply(int,void far *,int,int);
int sDisableInt(void);
int sEnableInt(void);
#else
int sControllerExists(int);
void sInitCriticalPtrs(CRITPTR_T far *);
LINE_T far *sLnNumToLnPtr(int,int);
void sFreeGPSDB(int,int);
void sFreeTxSDB(int,LINE_T *,WORD_T);
void sFreeRxSDB(int,LINE_T *,WORD_T);
WORD_T sSDBAddrToWindowNumber(int,WORD_T);

void sInit(int,int);
void sStartCP(int,int);
int  sGetNumLines(int);
void sInstallEvSM(int, WORD_T, void (*)(int, void far *));
void sInstallEvIFG(int, WORD_T, void (*)(int,unsigned int));
void sInitWinDesc(DPWINDOWS_T *,WORD_T,WORD_T,WORD_T);
char far *sMemAbs2Ptr(unsigned long);
void sInitCtlDesc(int,int,long,int,int,int,int,int,int,int,int);
void sInitCtlRegHSTi(int);
void sInitCtlRegH186(int);
void sCtlReset(int);
void sCtlInterrupt(int);
unsigned int sGetCtlID(int);
void sSlideWindow(int,int);
unsigned long sGetCtlTime(int);
#if QNX
void IntHandler();
pid_t far IntHand();
#else
void __interrupt __far IntHandler();
#endif
void sHookIsr(int);
void sUnHookIsr(void);
ULONG_T sFarPtrToAbs(void far *);
void far *sAbsToFarPtr(ULONG_T);
BOOL_T sSendCtlQMsg(int,void far *, void far *,int);
void sGetSysQMsg(int);
WORD_T sGetCtlQCnt(int);
WORD_T sGetSysQCnt(int);
UWORD_T sDecrementSFG(int,BYTE_T);
UWORD_T sIncrementSFG(int,BYTE_T);
UWORD_T sGetSFG(int,BYTE_T,UWORD_T);
UWORD_T sTestSetSFG(int,BYTE_T,UWORD_T);
void sGetDPAccess(int);
UBYTE_T sGetIFGStat(int, BYTE_T,UWORD_T far *);
void sDispatchIFG(int,int);
WORD_T sAllocGPSDB(int);
void sEnqSDB(int,WORD_T far *,WORD_T);
WORD_T sDeqSDB(int,WORD_T far *);
WORD_T sAllocTxSDB(int,LINE_T far *);
WORD_T sAllocRxSDB(int,LINE_T far *);
int sGetRxSDBIndex(int,int);
int sGetRxData(int,LINE_T far *, int, char far *);
int sPeekRx(LINE_T far *);
int sPutTxBlocks(int,LINE_T far *, int, char far *, int, char far *, int, char far *);
int sPutTxData(int,LINE_T far *, int, char far *);
void sEvSM00(int,void far *);
void sEvSM01(int,void far *);
void sEvIFGNull(int,unsigned int);
int sGetReplyID(int);
int sWaitForReply(int,void far *,int,int);
void sCleanupRx(int,int);
void sDelay (int);
int sDisableInt(void);
int sEnableInt(void);
void sSetNumLines(int,int);
void sClrSFG(int,BYTE_T,UWORD_T);
void sInitSFG(int,BYTE_T,UWORD_T);
void sSetSFG(int,BYTE_T,UWORD_T);
#endif




/*
*****************************************************************************
  Macros for sl_ssci.c
*****************************************************************************
*/
/*
*****************************************************************************
  Function:   sDisableInt

  Purpose:    Turn off system interrupts

  Call:       sDisableInt()

  Return:     void

  Comments:   This macro will turn off interrupts on your system, and is
              in effect the assembly instruction CLI.

  Warnings:   Implemented as a macro.  If you call this macro, you will
              not receive any more interrupts until you also call 
              sEnableInt().

*****************************************************************************
*/
#if BORLANDC
#define sDisableInt() asm  \
                   {      \
                    cli \
                   }
#elif QNX
#define sDisableInt() _disable();
#else
#define sDisableInt() __asm  \
                   {      \
                    __asm cli \
                   }
#endif

/*
*****************************************************************************
  Function:   sOutB

  Purpose:    Write a byte to I/O space   

  Call:       sOutB(io,Value)
              IOADD_T io:      I/O address to write to
              UBYTE_T Value:   Value to write      

  Return:     void

  Comments:   This macro outputs a byte to an I/O port.  It is intended to
              be compiler independent.

  Warnings:   Implemented as a macro.

*****************************************************************************
*/
#if BORLANDC
void sOutB(int,unsigned char);
#define sOutB(Io,Val) outportb((Io),(Val))
#else
void sOutB(unsigned,int);
#define sOutB(Io,Val) outp((Io),(Val))
#endif

/*
*****************************************************************************
  Function:   sOutW

  Purpose:    Write a word to I/O space   

  Call:       sOutW(io,Value)
              IOADD_T io:      I/O address to write to
              UWORD_T Value:   Value to write      

  Return:     void

  Comments:   This macro outputs a word to an I/O port.  It is intended to
              be compiler independent.

  Warnings:   Implemented as a macro.

*****************************************************************************
*/
#if BORLANDC
void sOutW(int,int);
#define sOutW(Io,Val) outport((Io),(Val))  
#else
void sOutW(unsigned,unsigned);
#define sOutW(Io,Val) outpw((Io),(Val))  
#endif

/*
*****************************************************************************
  Function:   sInB

  Purpose:    Reads a byte from I/O space   

  Call:       sInB(io)
              IOADD_T io:    I/O address to write to

  Return:     UBYTE_T:       Value read

  Comments:   This macro reads a byte from an I/O port.  It is intended to
              be compiler independent.

  Warnings:   Implemented as a macro.

*****************************************************************************
*/
#if BORLANDC
unsigned char sInB(int);
#define sInB(Io) inportb((Io))
#else
int sInB(unsigned);
#define sInB(Io) inp((Io))
#endif

/*
*****************************************************************************
  Function:   sInW

  Purpose:    Reads a word from I/O space   

  Call:       sInW(io)
              IOADD_T io:    I/O address to write to

  Return:     UWORD_T:       Value read

  Comments:   This macro reads a word from an I/O port.  It is intended to
              be compiler independent.

  Warnings:   Implemented as a macro.

*****************************************************************************
*/

unsigned sInW(unsigned);
#define sInW(Io) inpw((Io))


/*
*****************************************************************************
  Function:   sCopyBuf

  Purpose:    Copy a data buffer from one location in memory to another.

  Call:       sCopyBuf(SrcP,DstP,Cnt)
              void far *SrcP:      Ptr to source buffer
              void far *DstP:      Ptr to destination buffer
              UWORD_T Cnt:         Number of bytes to write

  Return:     void

  Comments:   Implemented as a macro.

  Warnings:   64k is the maximum amount of data that can be copied.

*****************************************************************************
*/
void sCopyBuf(char far *,char far *,int);
#define sCopyBuf(SrcP,DstP,Cnt) _fmemcpy(DstP,SrcP,Cnt);


/*
*****************************************************************************
  Function:   sEnableDPM

  Purpose:    sEnable Dual Port Memory

  Call:       EnableDPM(IOBase)
              IOADD_T IOBase:   Controller I/O base address

  Return:     void

  Comments:   Implemented as a macro.  In order to use Dual Port Memory, it
              must be enabled.  This call is used when a controller is
              first initialized and also after every slide of a memory
              window on a controller, such as in sSlideWindow().

  Warnings:   The control register index will be left pointing to the window
              control register.

*****************************************************************************
*/
void sEnableDPM(int);
#define sEnableDPM(IOBase) sOutB((IOBase)+IOX,CR3X|EN_RAM)

/*
*****************************************************************************
  Function:   sDisableDPM

  Purpose:    Disable dual port memory

  Call:       sDisableDPM(IOBase)
              IOADD_T IOBase:      Controller I/O base address

  Return:     void

  Comments:   Implemented as a macro.

  Warnings:   The control register index will be left pointing to the window
              control register.

*****************************************************************************
*/
void sDisableDPM(int);
#define sDisableDPM(IOBase) sOutB((IOBase)+IOX,CR3X|DIS_RAM)

/*
*****************************************************************************
  Function:   sSetPriorityHi

  Purpose:    Set system priority to the highest possible level.  

  Call:       sSetPriorityHi()

  Return:     int: The previous priority level, if supported by the operating
              environment, else 0.

  Comments:   Implemented as a macro.  This macro will set the current 
              task to the highest priority level in the system and will
              keep it there until sRestorePriority() is called.

  Warnings:   Must be modified for target operating enviornment. This only 
              needs to be used by multitasking operating systems that use
              multiple priority levels for scheduling tasks.

*****************************************************************************
*/
int sSetPriorityHi(void);
#define sSetPriorityHi() 0             /* DOS has no such mechanism */
                        

/*
*****************************************************************************
  Function:   sRestorePriority

  Purpose:    Restores system priority to some previous level.

  Call:       sRestorePriority()
              int Level;   Priority level to restore to.

  Return:     void

  Comments:   Implemented as a macro.  This macro will restore the 
              priority level back to the level it was at before the
              sSetPriorityHi() call was made.

  Warnings:   Must be modified for target operating enviornment. This only 
              needs to be used by multitasking operating systems that use
              multiple priority levels for scheduling tasks.

*****************************************************************************
*/
void sRestorePriority(int Level);
#define sRestorePriority(Level) Level=Level

/*
*****************************************************************************
  Function:   sRelDPAccess

  Purpose:    Release Dual Port memory access.

  Call:       sRelDPAccess()

  Return:     void

  Comments:   Implemented as a macro.  This macro will gain exclusive access
              to Dual Port Memory for the currenly executing code.  This 
              is essential to prevent race conditions between the system
              and controller at critical times.

*****************************************************************************
*/
#define sRelDPAccess(CtlNum) *Ctl[CtlNum].SysDPAccP = FALSE


/*
*****************************************************************************
  Function:   sFreeGPSDB

  Purpose:    Free a shared data buffer by returning to the free pool.

  Call:       sFreeGPSDB(CtlNum,N)
              int CtlNum:    Controller index number
              int N:         Shared Data Buffer index number

  Return:     void

  Comments:   Implemented as a macro.
              Since both processors use the same SDB free pool the free
              operation is protected by sDisableInt() and sGetDPAccess().
              Before returning, sRelDPAccess() and sEnableInt() are done.

*****************************************************************************
*/
#define sFreeGPSDB(CtlNum,N)     sDisableInt(); \
                          sGetDPAccess(CtlNum); \
                          Ctl[CtlNum].SDBTbl[N].Next = *Ctl[CtlNum].SDBFreeP; \
                          *Ctl[CtlNum].SDBFreeP = (N); \
                          *Ctl[CtlNum].SDBFreeCntP += 1; \
                          sRelDPAccess(CtlNum); \
                          sEnableInt()


/*
*****************************************************************************
  Function:   sFreeTxSDB

  Purpose:    Free a SDB (shared data buffer) that was used for Tx on a line,
              return the SDB to the free pool.

  Call:       sFreeTxSDB(CtlNum,n,LnP)
              int CtlNum:       Controller index number
              LINE_T far *LnP:  Ptr to the line descriptor of the line that
                                the SDB will be used on 
              WORD_T n:         SDB to be freed 

  Return:     void

  Comments:   Implemented as a macro.

  Warnings:   If the SDB being freed was not used on a serial line, do not
              use this function.  Instead use sSDBFree() to free the SDB.
              sSDBFree() does not change the count of SDBs that are
              attached to a line.
  
*****************************************************************************
*/
#define sFreeTxSDB(CtlNum,LNP,N) sDisableInt(); \
                        sGetDPAccess(CtlNum); \
                        if(--(LNP)->TxSDBCnt < 0) \
                           (LNP)->TxSDBCnt = 0; \
                        Ctl[CtlNum].SDBTbl[N].Next = *Ctl[CtlNum].SDBFreeP; \
                        *Ctl[CtlNum].SDBFreeP = (N); \
                        *Ctl[CtlNum].SDBFreeCntP += 1; \
                        sRelDPAccess(CtlNum); \
                        sEnableInt()

/*
*****************************************************************************
  Function:   sFreeRxSDB

  Purpose:    Free a SDB (shared data buffer) that was used for Rx on a line,
              return the SDB to the free pool.
 
  Call:       sFreeRxSDB(CtlNum,n,LnP)
              int CtlNumz:         Controller index number
              WORD_T n:            SDB to be freed
              LINE_T far *LnP:     Ptr to the line descriptor of the line 
                                   that the SDB will be used on
                                                        
  Return:     void

  Comments:   Implemented as a macro.

  Warnings:   If the SDB being freed was not used on a serial line, do not
              use this function.  Instead use sSDBFree() to free the SDB.
              sSDBFree() does not change the count of SDBs that are
              attached to a line.
  
*****************************************************************************
*/
#define sFreeRxSDB(CtlNum,LNP,N) sDisableInt(); \
                        sGetDPAccess(CtlNum); \
                        if(--(LNP)->RxSDBCnt < 0) \
                           (LNP)->RxSDBCnt = 0; \
                        Ctl[CtlNum].SDBTbl[N].Next = *Ctl[CtlNum].SDBFreeP; \
                        *Ctl[CtlNum].SDBFreeP = (N); \
                        *Ctl[CtlNum].SDBFreeCntP += 1; \
                        sRelDPAccess(CtlNum); \
                        sEnableInt()

/*
 *****************************************************************************
  Function:   sSlideWindow

  Purpose:    Slide to the correct memory address on the controller

  Call:       void sSlideWindow(CtlNum,offset)
              int CtlNum;      Controller index number
              int offset;      which window in DPM to set the controller to 

  Return:     void

  Comments:   This function will slide the Dual Port Memory (DPM) window to
              the 16k window defined by 'offset'.  'Offset' is an integer
              index corresponding to each 16k block of memory on the 
              controller.  For example:
  
              offset      controller memory block address
              ------      -------------------------------
                0           0k
                1          16k  
                2          32k
                3          48k
                etc.       etc.

 *****************************************************************************
*/
#ifdef JAMMIN
#define sSlideWindow(CtlNum,Offset) sOutB(Ctl[CtlNum].Io+IOX,CR3X | DIS_RAM); \
                               sOutB(Ctl[CtlNum].Io,Offset);  \
                               EnableDPM(Ctl[CtlNum].Io);    /* enable DPM */
#endif

/*
*****************************************************************************
  Function:   sSDBAddrToWindowNumber

  Purpose:    Convert a controller address to a 16k window number

  Call:       sSDBAddrToWindow(CtlNum,N)
              int CtlNum:   Controller index number
              int N:        Shared Data Buffer index number

  Return:     long

  Comments:   Implemented as a macro.  This macro converts a controller
              address into a window index number that corresponds to a 16k
              block of memory on a controller.

*****************************************************************************
*/
#define sSDBAddrToWindowNumber(CtlNum,N) (WORD_T)(((long)Ctl[CtlNum].SDBTbl[N].BufOff << 4) / 0x4000);

/*
*****************************************************************************
  Function:   sLnNumToLnPtr

  Purpose:    Convert a device number to a pointer to a line table.

  Call:       sLnNumToLnPtr(CtlNum,dev)
              int CtlNum:    Controller index number
              int dev:       Device number

  Return:     LINE_T far *

  Comments:   Implemented as a macro.  This macro converts a line number
              to a pointer to a line table descriptor.

*****************************************************************************
*/
#define sLnNumToLnPtr(CtlNum,dev) aaCritPtrs.sCtlArrP[CtlNum].Line[dev];


#endif /* #ifndef S_API */


